/****************************************************************************
**
** Copyright (C) 2018 The Qt Company Ltd.
** Contact: http://www.qt.io/licensing/
**
** This file is part of Qt 3D Studio.
**
** $QT_BEGIN_LICENSE:GPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 or (at your option) any later version
** approved by the KDE Free Qt Foundation. The licenses are as published by
** the Free Software Foundation and appearing in the file LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef Q3DSOBJECT3D_P_H
#define Q3DSOBJECT3D_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <Qt3DStudioRuntime2/private/q3dsruntimeglobal_p.h>
#include <QQuickItem>
#include <private/q3dsuippresentation_p.h>

QT_BEGIN_NAMESPACE

class Q3DSLayer3D;

class Q3DSV_PRIVATE_EXPORT Q3DSObject3D : public QQuickItem
{
    Q_OBJECT

public:
    Q3DSObject3D();
    ~Q3DSObject3D();

    virtual Q3DSGraphObject *createObject(Q3DSUipPresentation *presentation) = 0;

    Q3DSGraphObject *object() const { return m_object; }
    Q3DSLayer3D *layer() const { return m_layer; }

    enum InvalidateFlag {
        InvalidateObject = 0x01,
        InvalidateLayer = 0x02
    };
    Q_DECLARE_FLAGS(InvalidateFlags, InvalidateFlag)

    void invalidate(InvalidateFlags flags);

protected:
    void itemChange(ItemChange, const ItemChangeData &) override;
    QByteArray makeIdAndName() const;
    virtual bool propertyChanged(const QMetaProperty &property,
                                 Q3DSGraphObject *target,
                                 const QMetaProperty &targetProperty);

private:
    void activate();
    void activate_helper(Q3DSLayer3D *layer);
    void setLayer_recursive(Q3DSLayer3D *layer);

    Q3DSGraphObject *m_object = nullptr;
    Q3DSLayer3D *m_layer = nullptr;
    Q3DSObject3D *m_pendingObjectParent = nullptr;
    bool m_hasPendingActivate = false;
    QMetaObject::Connection m_layerConn;

    friend class Q3DSLayer3D;
    friend class Q3DSPropertyBinder;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(Q3DSObject3D::InvalidateFlags)

QT_END_NAMESPACE

#endif // Q3DSOBJECT3D_P_H
